% optimizes macro-f1 by solving equivalent cost-sensitive
% problem. See "Optimizing F-Measures by Cost-Sensitive Classification".
function [w, b] = cost_sensitive(train_data, options)
    tmp = [strfind(options, '-C') strfind(options, '-v') strfind(options, '-c')];
    if ~isempty(tmp)
        error('options may not contain -C, -c or -v: "%s"', options)
    end

    quiet = 0;
    if strfind(options, '-q')
        quiet = 1;
    end

    x = train_data.x;
    y = train_data.y;
    map = train_data.map;

    d = size(y, 2);
    w = zeros(size(x, 1), d);
    b = 0;
    eps0 = 0.1;

    for i = 1:d
        fprintf('INFO: training label %d (i %d)...\n', map(i), i);
        t0 = clock;

        bestScore = -Inf;
        for t = eps0:eps0:1
            a = (2 - t) / t;

            if ~quiet
                fprintf('a = %.3f \tt = %.2f\n', a, t);
            end

            cv_options = [options ' -C -v 3 -w1 ' num2str(a)];
            m = train(2*y(:, i)-1, x, cv_options, 'col', 'fscore');

            if bestScore < m(3)
                bestScore = m(3);
                bestC = m(1);
                bestA = a;
            end
        end

        final_options = [options ' -c ' num2str(bestC) ' -w1 ' num2str(bestA)];
        m = train(2*y(:, i)-1, x, final_options, 'col', '');
        w(:, i) = m.w';

        if ~quiet
            fprintf('INFO: using C = %g a = %g with score = %g\n', bestC, bestA, bestScore);
        end
        fprintf('INFO: training time: %.6f seconds\n', etime(clock, t0));
    end

