% optimizes micro-f1 by solving equivalent cost-sensitive
% problem. See "Optimizing F-Measures by Cost-Sensitive Classification".
function [w, b] = simple_cost_sensitive_micro_7_1(train_data, options)
    tmp = [strfind(options, '-C') strfind(options, '-v') strfind(options, '-c')];
    if ~isempty(tmp)
        error('options may not contain -C, -c or -v: "%s"', options)
    end

    quiet = 0;
    if strfind(options, '-q')
        quiet = 1;
    end

    x = train_data.x;
    y = train_data.y;
    map = train_data.map;

    perm = randperm(size(y, 1))';

    d = size(y, 2);
    w = zeros(size(x, 1), d);
    b = 0;

    bestScore = -Inf;
    % approx t = (1:7) / 7
    for a = [1 1.33 1.8 2.5 3.67 6 13]
        for c = [1]
            tp = 0;
            fn = 0;
            fp = 0;
            for i = 1:d
                cv_options = [options ' -c ' num2str(c) ' -w1 ' num2str(a)];
                [ltp, lfn, lfp] = cross_validate(y(:,i), x, cv_options, perm);
                tp = tp + ltp;
                fn = fn + lfn;
                fp = fp + lfp;
            end
            score = 2*tp / (2*tp + fn + fp);
            if bestScore < score
                bestScore = score;
                bestA = a;
                bestC = c;
            end
        end
    end

    final_options = [options ' -c ' num2str(bestC) ' -w1 ' num2str(bestA)];
    for i = 1:d
        m = train(2*y(:, i) - 1, x, final_options, 'col', '');
        w(:, i) = m.w';
    end

    if ~quiet
        fprintf('INFO: using C = %g a = %g with score = %g\n', bestC, bestA, bestScore);
    end

function [tp, fn, fp] = cross_validate(y, x, options, perm)
    nr_fold = 3;
    l = size(perm, 1);

    p = zeros(size(y));
    for fold = 1:nr_fold
        train_id = [1:floor((fold - 1)*l/nr_fold) floor(fold*l/nr_fold) + 1:l]';
        train_id = perm(train_id);
        valid_id = [floor((fold - 1)*l/nr_fold) + 1:floor(fold*l/nr_fold)]';
        valid_id = perm(valid_id);

        m = train(2*y(train_id)-1, x(:, train_id), options, 'col', '');
        p(valid_id) = (m.w*x(:, valid_id))';
    end

    y = 2*y - 1;
    p = 2*(p > 0) - 1;
    tp = full(sum(y == +1 & p == +1));
    fn = full(sum(y == +1 & p == -1));
    fp = full(sum(y == -1 & p == +1));