% Calculates macro/micro/instance-f1 score.
% y is the ground truth.
% p is the prediction.
% y and p must have dimensions number of instances * number of classes.
% y and p must be +1/-1 matrices. 
function [macro, micro, instance] = fscore(y, p)
    tp = (y == +1 & p == +1);
    fn = (y == +1 & p == -1);
    fp = (y == -1 & p == +1);

    % ltp, lfn, lfp have dimensions 1 * number of labels
    ltp = sum(tp);
    lfn = sum(fn);
    lfp = sum(fp);

    macro = sum(2*ltp ./ (2*ltp + lfp + lfn), 'omitnan') / size(y, 2);
    micro = 2*sum(ltp) / sum(2*ltp + lfp + lfn);
    if isnan(micro)
        micro = 0;
    end

    % itp, ifn, ifp have dimensions number of instances * 1
    itp = sum(tp, 2);
    ifn = sum(fn, 2);
    ifp = sum(fp, 2);
    instance = sum(2*itp ./ (2*itp + ifp + ifn), 'omitnan') / size(y, 1);
