% Calculates ranking loss.
% y is the ground truth.
% p is the decision values.
% y and p must have dimensions number of instances * number of classes.
% y must be +1/-1 matrices.
% Any instance with identically 1/-1 labels are considered to have zero loss
function [rloss] = ranking_loss(y, p)
    [dummy, ord] = sort(p, 2, 'descend');
    [dummy, rank] = sort(ord, 2);
    d = size(y, 2);
    k = sum(y, 2);
    rloss = sum(rank .* (y > 0), 2) - k .* (k+1) / 2;
    rloss = rloss ./ (k .* (d - k));
    rloss(isinf(rloss)) = 0;
    rloss(isnan(rloss)) = 0;
    rloss = mean(rloss);