# On the Use of Unrealistic Predictions in Hundreds of Papers Evaluating Graph Representations

This repository contains the code for the downstream multilabel classification task for the paper "On the Use of Unrealistic Predictions in Hundreds of Papers Evaluating Graph Representations".

## Installation
Run under matlab or octave

```matlab
make
multilabel_make
```

## Usage
Under matlab or octave, run

```matlab
unified_run(train_path, test_path, options, method)
```

Where
+ `train_path` and `test_path` is path to multilabel svm format files.
  + They must start with `data/`
  + They must end with `trainN.svm` and `testN.svm` where `N` is a single digit number.
+ `options` is the multicore liblinear options string.
+ `method` may be one of the following
  + `unrealistic`
  + `one-vs-rest`
  + `one-vs-rest-C`
  + `one-vs-rest-no-empty`
  + `thresholding`
  + `cost-sensitive`
  + `cost-sensitive-simple`
  + `cost-sensitive-simple-micro`
+ `method` may be prefixed with `resume-`, which loads previously trained model instead of training.
  + `options` string must match with previous run or be empty.

Notice that for `cost-sensitive-simple` and `cost-sensitive-simple-micro`, if you want to check different grid values of (C, t) pairs, you need to modify the code therein.
For example, in `cost-sensitive-simple`, you can replace a by a = [1 1.4 2 3 5 11] if you want to check t = {1/6, 2/6, ..., 1}.
And you can replace c = [1] by c = [0.01 0.1 1 2] if you want to check these regularization parameters.

### Example
```matlab
unified_run('data/deepwalk/blogcatalog/train0.svm', 'data/deepwalk/blogcatalog/test0.svm', '-s 0 -e 0.0001 -m 12 -q', 'unrealistic')
```

### Running `cost-sensitive-no-empty`
Run `cost-sensitive` first. Then under matlab or octave, run
```matlab
csne(train_path, test_path)
```
which reuses the model from `cost-sensitive`.

## Training/test Data Splits
You can either directly download the five 80/20 splitted datasets from [embedding_data/data.tar.xz](https://www.csie.ntu.edu.tw/~cjlin/papers/multilabel-embedding/embedding_data/data.tar.xz) or run 
```bash
python3 split.py src/file.svm dst/dir/ 0.8
    ```
which randomly splits `src/file.svm` into 80/20 training/testing splits 5 times to `dst/dir/`.
(Notice that in split.py we have fixed the seed so the splitted datasets should be the same in different runs.)

### Example
```bash
mkdir -p data/deepwalk/blogcatalog
python3 split.py ../../embed_gen/exp/blogcatalog.1.dw.hs0.svm data/deepwalk/blogcatalog/ 0.8
```
which will give you
```
data/deepwalk/blogcatalog/train0.svm
data/deepwalk/blogcatalog/train1.svm
data/deepwalk/blogcatalog/train2.svm
data/deepwalk/blogcatalog/train3.svm
data/deepwalk/blogcatalog/train4.svm
data/deepwalk/blogcatalog/test0.svm
data/deepwalk/blogcatalog/test1.svm
data/deepwalk/blogcatalog/test2.svm
data/deepwalk/blogcatalog/test3.svm
data/deepwalk/blogcatalog/test4.svm
```
