# On the Use of Unrealistic Predictions in Hundreds of Papers Evaluating Graph Representations

This repository contains the code for the upstream graph representation learning task for the paper "On the Use of Unrealistic Predictions in Hundreds of Papers Evaluating Graph Representations".

## Getting Started

**Step 0.** Setup your docker and run:
```
mkdir -p exp
docker-compose up
```
*Note.* By default that Unix socket is owned by the user **root** and other users can only access it using `sudo`.

*Ref.* [HERE](https://docs.docker.com/engine/install/linux-postinstall/)

**Step 1.** Download the data via:
```
docker exec -it embed_gen bash datasets/get_data.sh
```

**Step 2.** Generate the embeddings and SVM files with the scripts:
```
docker exec -it embed_gen bash script/exp_dw.sh 'blogcatalog flickr ppi youtube'
docker exec -it embed_gen bash script/exp_line.sh 'blogcatalog flickr ppi youtube'
docker exec -it embed_gen bash script/exp_n2v.sh 'blogcatalog flickr ppi youtube'
```
*Note.* Please find the modeling parameters here, and change the `WORKER` variable to run the model(s) in parallel

**Results.**

Generated files are saved in the `exp` folder. 

Take the **blogcatalog** as the example. You will get
- two DeepWalk's embedding files
  - `exp/blogcatalog.1.dw.hs0` (Negative Sampling)
  - `exp/blogcatalog.1.dw.hs1` (Hierarchical Softmax)
- two DeepWalk's SVM files
  - `exp/blogcatalog.1.dw.hs0.svm`
  - `exp/blogcatalog.1.dw.hs1.svm`
- four node2vec's embedding files
  - `exp/blogcatalog.1.pyn2v.p0.25.q0.25` (p=0.25, q=0.25)
  - `exp/blogcatalog.1.pyn2v.p0.25.q4` (p=0.25, q=4)
  - `exp/blogcatalog.1.pyn2v.p4.q0.25` (p=4, q=0.25)
  - `exp/blogcatalog.1.pyn2v.p4.q4` (p=4, q=4)
- four nod2vec's SVM files
  - `exp/blogcatalog.1.pyn2v.p0.25.q0.25.svm`
  - `exp/blogcatalog.1.pyn2v.p0.25.q4.svm`
  - `exp/blogcatalog.1.pyn2v.p4.q0.25.svm`
  - `exp/blogcatalog.1.pyn2v.p4.q4.svm`
- three LINE's embedding files
  - `exp/blogcatalog.1.line1` (1st embedding)
  - `exp/blogcatalog.1.line2` (2nd embedding)
  - `exp/blogcatalog.1.line` (1st&2nd concatenation)
- one LINE's SVM file
  - `exp/blogcatalog.1.line.svm`


## Testing Environment of the Paper Work
- Python 3.8.10
- g++ (Ubuntu 9.3.0-17ubuntu1~20.04) 9.3.0

## Source of the Tools
- [deepwalk](https://github.com/phanein/deepwalk)
- [node2vec](https://github.com/aditya-grover/node2vec)
- [LINE](https://github.com/tangjianpku/LINE)
