import os, sys
import random
from argparse import ArgumentParser
from collections import defaultdict
import numpy as np
import pandas as pd

parser = ArgumentParser(description='Argument Parser')
parser.add_argument("--embed", required=True, help='file path to the embeddings')
parser.add_argument("--label", required=True, help='file path to the labels')
parser.add_argument("--save", required=True, help='file path for saving results')
parser.add_argument("--train_size", required=False, type=float, default=[0.8],
                    nargs='+', help='train size for exp.')
parser.add_argument("--times", required=False, type=int, default=5, help='number of exp. times')
args = parser.parse_args()

embeddings, labels = {}, defaultdict(list)

print(f'load {args.embed}')
with open(args.embed, 'r') as f:
    next(f)
    for line in f:
        line = line.rstrip('\n').split(' ')
        index, embed = line[0], line[1:]
        embeddings[index] = ' '.join([f'{i}:{e}' for i, e in enumerate(embed, 1)])

print(f'load {args.label}')
num_labels = {}
with open(args.label, 'r') as f:
    for line in f:
        index, label = line.rstrip('\n').split('\t')
        labels[index].append(label)
        num_labels[label] = 1
num_labels = len(num_labels)
print(f'unique index:\t{len(labels)}')
print(f'unique label:\t{num_labels}')

# create label-fea. data
data = []
for index in labels:
    if index in embeddings:
        y = ','.join(labels[index])
        data.append(f"{y} {embeddings[index]}")

# train/test split here
for train_size in args.train_size:
    random.seed(0)
    indexes = list(labels.keys())
    print(f'given train_size={train_size}')

    if train_size == 1.0:
        print(f'save to {args.save}')
        with open(args.save, 'w') as f:
            f.write('\n'.join(data))
            f.write('\n')
        break

    for exp in range(1, args.times+1):
        random.shuffle(data)
        train_cut = int(len(indexes)*train_size)
        train_data = data[:train_cut]
        test_data = data[train_cut:]
        print(f'train size:\t{len(train_data)}, test size:\t{len(test_data)}')

        train_data_path = f"{args.save}-{train_size}-{exp}.train"
        test_data_path = f"{args.save}-{train_size}-{exp}.test"
        print(f'save to {train_data_path} & {test_data_path}')
        with open(train_data_path, 'w') as f:
            f.write('\n'.join(train_data))
            f.write('\n')
        with open(test_data_path, 'w') as f:
            f.write('\n'.join(test_data))
            f.write('\n')

