#!/usr/bin/env python

"""preprocess_label.py: Script for preprocessing graph data. This code is written by modifying the code in DeepWalk GitHub (https://github.com/phanein/deepwalk/blob/master/example_graphs/scoring.py)."""

import numpy
import sys

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
from collections import defaultdict
from six import iteritems
from scipy.io import loadmat

def sparse2graph(x):
    G = defaultdict(lambda: set())
    cx = x.tocoo()
    for i,j,v in zip(cx.row, cx.col, cx.data):
        G[i].add(j)
    return {str(k): [str(x) for x in v] for k,v in iteritems(G)}

def main():
  parser = ArgumentParser("preprocess",
                          formatter_class=ArgumentDefaultsHelpFormatter,
                          conflict_handler='resolve')
  parser.add_argument("--matfile", required=True,
                      help='A .mat file containing the adjacency matrix and node labels of the input network.')
  parser.add_argument("--adj-matrix-name", default='network',
                      help='Variable name of the adjacency matrix inside the .mat file.')
  parser.add_argument("--label-matrix-name", default='group',
                      help='Variable name of the labels matrix inside the .mat file.')

  args = parser.parse_args()

  mat = loadmat(args.matfile)[args.adj_matrix_name]
  graph = sparse2graph(mat)
  print(graph)

  import sys
  sys.exit()

if __name__ == "__main__":
  sys.exit(main())
